// Generated by CoffeeScript 1.3.3
(function() {
  var EntityManager;

  Artemis.EntityManager = {};

  EntityManager = (function() {

    function EntityManager(world) {
      this.world = world;
      this.entities = {};
      this.componentsByType = {};
      this.nextId = 0;
      this.count = 0;
      this.totalCreated = 0;
    }

    EntityManager.prototype._create = function() {
      var entity;
      entity = new Artemis.Entity(this.world, this.nextId);
      this.nextId++;
      this.entities[entity.id] = entity;
      this.count++;
      this.totalCreated++;
      return entity;
    };

    EntityManager.prototype._getEntity = function(id) {
      return this.entities[id];
    };

    EntityManager.prototype.refresh = function(entity) {
      var system, _i, _len, _ref, _results;
      _ref = this.world.systemManager.allSystems;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        system = _ref[_i];
        _results.push(system.change(entity));
      }
      return _results;
    };

    EntityManager.prototype.remove = function(entity) {
      delete this.entities[entity.id];
      this._removeAllComponents(entity);
      this.count--;
      return entity = null;
    };

    EntityManager.prototype._isActive = function(id) {
      if (this.entities[id]) {
        return true;
      } else {
        return false;
      }
    };

    EntityManager.prototype._addComponent = function(entity, component) {
      var componentType, components;
      componentType = Artemis.ComponentTypeManager.getType(component);
      components = this.componentsByType[componentType.id];
      if (!components) {
        components = {};
        this.componentsByType[componentType.id] = components;
      }
      components[entity.id] = component;
      return entity._addTypeBit(componentType.bit);
    };

    EntityManager.prototype._removeComponent = function(entity, componentName) {
      var componentType, components;
      componentType = Artemis.ComponentTypeManager.getTypeByName(componentName);
      components = this.componentsByType[componentType.id];
      delete components[entity.id];
      return entity._removeTypeBit(componentType.bit);
    };

    EntityManager.prototype._getComponent = function(entity, componentName) {
      var componentType, components;
      componentType = Artemis.ComponentTypeManager.getTypeByName(componentName);
      components = this.componentsByType[componentType.id];
      if (components) {
        return components[entity.id];
      } else {
        return null;
      }
    };

    EntityManager.prototype._getAllComponents = function(entity) {
      var allComponents, component, components, index, _ref;
      allComponents = [];
      _ref = this.componentsByType;
      for (index in _ref) {
        components = _ref[index];
        if (components) {
          component = components[entity.id];
          if (component) {
            allComponents.push(component);
          }
        }
      }
      return allComponents;
    };

    EntityManager.prototype._removeAllComponents = function(entity) {
      var componentTypes, index, _ref, _results;
      _ref = this.componentsByType;
      _results = [];
      for (index in _ref) {
        componentTypes = _ref[index];
        if (entity.id in componentTypes) {
          delete componentTypes[entity.id];
          if (Object.keys(componentTypes).length === 0) {
            _results.push(delete this.componentsByType[index]);
          } else {
            _results.push(void 0);
          }
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    return EntityManager;

  })();

  Artemis.EntityManager = EntityManager;

}).call(this);
